/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package org.ow2.petals.tools.jmx.api;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.management.MBeanServerConnection;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

import org.ow2.petals.tools.jmx.api.exception.ConnectionErrorException;

/**
 * A Petals JMX Server Connection
 *
 * @author cdeneux - Capgemini Sud
 */
public final class PetalsJMXConnection {

    private String host = null;

    private Integer port = null;

    private String username = null;

    private String password = null;

    private JMXConnector connector = null;

    /**
     * Creates a new instance of {@link PetalsJMXConnection}
     *
     * @param host
     * @param port
     * @param username
     * @param password
     * @throws ConnectionErrorException
     */
    public PetalsJMXConnection(String host, Integer port, String username,
        String password) throws ConnectionErrorException {
        this.host = host;
        this.port = port;
        this.username = username;
        this.password = password;

        String urlStr =null;

        try {
            Properties env = System.getProperties();
            env.put("java.naming.factory.initial",
                "com.sun.jndi.rmi.registry.RegistryContextFactory");
            Map<String, Object> args = new HashMap<String, Object>();

            // set credentials
            if (!"".equals(this.username)) {
                if (this.password == null) {
                    this.password = "";
                }
                String[] credentials = new String[] {this.username,
                    this.password};
                args.put(JMXConnector.CREDENTIALS, credentials);
            }

            urlStr = "service:jmx:rmi:///jndi/rmi://" + this.host + ":"
                + this.port + "/management/rmi-jmx-connector";
            JMXServiceURL url = new JMXServiceURL(urlStr);
            this.connector = JMXConnectorFactory.connect(url, args);
        } catch (MalformedURLException e) {
            throw new ConnectionErrorException(e);
        } catch (IOException e) {
            throw new ConnectionErrorException("Can not connect using URI : "+urlStr+" with "+username+"/"+password, e);
        }  catch (SecurityException e) {
            throw new ConnectionErrorException(e);
        }
    }

    public MBeanServerConnection getMBeanServerConnection() throws ConnectionErrorException {
        try {
            return this.connector.getMBeanServerConnection();
        } catch (IOException e) {
            throw new ConnectionErrorException(e);
        }
    }
}
